#include <stm32f10x.h>
#include "config.h"


GPIO_InitTypeDef GPIO_InitStructure;
USART_InitTypeDef USART_InitStructure;
TIM_TimeBaseInitTypeDef TIM_TimeBaseStructure;
TIM_OCInitTypeDef TIM_OCInitStructure;
NVIC_InitTypeDef NVIC_InitStructure;
ADC_InitTypeDef ADC_InitStructure;
DMA_InitTypeDef DMA_InitStructure;

//-----------------------------------------------------------------------------
//                           クロックの設定
//-----------------------------------------------------------------------------
#define HSIcal 0x0c
void RCC_Config()
{
	RCC_DeInit();
	// Enable Prefetch Buffer
	FLASH_PrefetchBufferCmd(FLASH_PrefetchBuffer_Enable);
	// Flash 2 wait state
	FLASH_SetLatency(FLASH_Latency_2);
	// HCLK = SYSCLK
	RCC_HCLKConfig(RCC_SYSCLK_Div1); 
	// PCLK2 = HCLK
	RCC_PCLK2Config(RCC_HCLK_Div1); 
	// PCLK1 = HCLK/2
	RCC_PCLK1Config(RCC_HCLK_Div2);
	// ADCCLK = 64 / 6 = 10.7MHz
	RCC_ADCCLKConfig(RCC_PCLK2_Div6);
	// PLLCLK = 4MHz * 16 = 64 MHz
	RCC_PLLConfig(RCC_PLLSource_HSI_Div2, RCC_PLLMul_16);
	RCC_AdjustHSICalibrationValue(HSIcal);
	// Enable PLL
	RCC_PLLCmd(ENABLE);
	// Wait till PLL is ready
	while(RCC_GetFlagStatus(RCC_FLAG_PLLRDY) == RESET);
	// Select PLL as system clock source
	RCC_SYSCLKConfig(RCC_SYSCLKSource_PLLCLK);
	// Wait till PLL is used as system clock source
	while(RCC_GetSYSCLKSource() != 0x08);
	
	SystemCoreClockUpdate();
}

//-----------------------------------------------------------------------------
//                           ベクタテーブル
//-----------------------------------------------------------------------------
void InitVectorTable(){
	NVIC_SetVectorTable(NVIC_VectTab_FLASH, 0x08000000);
}

//-----------------------------------------------------------------------------
//                           シリアル通信
//-----------------------------------------------------------------------------
#define TRX_BUFFER_SIZE 64 // 送受信バッファサイズ
struct {
    unsigned int head, end;
    char buff[TRX_BUFFER_SIZE];
} txb,rxb;

DMA_InitTypeDef DMA_InitStructure_TX;
DMA_InitTypeDef DMA_InitStructure_RX;
void SIO_Config()
{
	// クロックを有効にする
	RCC_AHBPeriphClockCmd(RCC_AHBPeriph_DMA1, ENABLE);
	RCC_APB2PeriphClockCmd(RCC_APB2Periph_USART1 | RCC_APB2Periph_GPIOA | RCC_APB2Periph_AFIO, ENABLE);
	
	// USART1用出力ポートの設定
	GPIO_StructInit(&GPIO_InitStructure);
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_9;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF_PP;
	GPIO_Init(GPIOA, &GPIO_InitStructure);
	
	// USART1
	USART_StructInit(&USART_InitStructure);
	USART_InitStructure.USART_BaudRate = 115200;
	USART_InitStructure.USART_WordLength = USART_WordLength_8b;
	USART_InitStructure.USART_StopBits = USART_StopBits_1;
	USART_InitStructure.USART_Parity = USART_Parity_No;
	USART_InitStructure.USART_HardwareFlowControl = USART_HardwareFlowControl_None;
	USART_InitStructure.USART_Mode = USART_Mode_Rx | USART_Mode_Tx;
	USART_Init(USART1, &USART_InitStructure);
	
	// USART用DMAの初期化 送信用
	#define USART1_DR_Base  0x40013804
	DMA_StructInit(&DMA_InitStructure_TX);
	DMA_InitStructure_TX.DMA_PeripheralBaseAddr = USART1_DR_Base;
	DMA_InitStructure_TX.DMA_MemoryBaseAddr = (uint32_t)txb.buff;
	DMA_InitStructure_TX.DMA_DIR = DMA_DIR_PeripheralDST;
	DMA_InitStructure_TX.DMA_BufferSize = 0;
	DMA_InitStructure_TX.DMA_PeripheralInc = DMA_PeripheralInc_Disable;
	DMA_InitStructure_TX.DMA_MemoryInc = DMA_MemoryInc_Enable;
	DMA_InitStructure_TX.DMA_PeripheralDataSize = DMA_PeripheralDataSize_Byte;
	DMA_InitStructure_TX.DMA_MemoryDataSize = DMA_MemoryDataSize_Byte;
	DMA_InitStructure_TX.DMA_Mode = DMA_Mode_Normal;
	DMA_InitStructure_TX.DMA_Priority = DMA_Priority_VeryHigh;
	DMA_InitStructure_TX.DMA_M2M = DMA_M2M_Disable;
	DMA_Init(DMA1_Channel4, &DMA_InitStructure_TX);
	// 受信用
	DMA_StructInit(&DMA_InitStructure_RX);
	DMA_InitStructure_RX.DMA_PeripheralBaseAddr = USART1_DR_Base;
	DMA_InitStructure_RX.DMA_MemoryBaseAddr = (uint32_t)rxb.buff;
	DMA_InitStructure_RX.DMA_DIR = DMA_DIR_PeripheralSRC;
	DMA_InitStructure_RX.DMA_BufferSize = TRX_BUFFER_SIZE;
	DMA_InitStructure_RX.DMA_PeripheralInc = DMA_PeripheralInc_Disable;
	DMA_InitStructure_RX.DMA_MemoryInc = DMA_MemoryInc_Enable;
	DMA_InitStructure_RX.DMA_PeripheralDataSize = DMA_PeripheralDataSize_Byte;
	DMA_InitStructure_RX.DMA_MemoryDataSize = DMA_MemoryDataSize_Byte;
	DMA_InitStructure_RX.DMA_Mode = DMA_Mode_Circular;
	DMA_InitStructure_RX.DMA_Priority = DMA_Priority_VeryHigh;
	DMA_InitStructure_RX.DMA_M2M = DMA_M2M_Disable;
	DMA_Init(DMA1_Channel5, &DMA_InitStructure_RX);
	// バッファクリア
	txb.head = txb.end = rxb.head = rxb.end = 0;
	// USARTによるDMAリクエストon
	USART_DMACmd(USART1, USART_DMAReq_Tx, ENABLE);
	USART_DMACmd(USART1, USART_DMAReq_Rx, ENABLE);
	// 受信DMA動作開始
	DMA_Cmd(DMA1_Channel5, ENABLE);
	
	// USART動作開始
	USART_Cmd(USART1, ENABLE);
}

// 送信バッファに一文字追加
// headに新しく追加し，endは次に送信されるデータ
// バッファに空きが無い場合は待機
void outbyte(char c)
{
	// バッファ内データをカウントし，空きがない場合待機
	volatile unsigned short remain;
	while(1){
		// この関数は多重に実行されるとまずいので割り込みをマスクする
		//mask
		// DMACを一時的に停止
		DMA_Cmd(DMA1_Channel4, DISABLE);
		
		remain = DMA_GetCurrDataCounter(DMA1_Channel4);
		if(remain != TRX_BUFFER_SIZE) break;
		
		// 割り込み解除
		//mask
		// DMAC動作再開
		DMA_Cmd(DMA1_Channel4, ENABLE);
		while((remain = DMA_GetCurrDataCounter(DMA1_Channel4)) == TRX_BUFFER_SIZE);
	}
	
	// 次に書き込もうとする領域がバッファ終端なら前に詰める
	if(txb.head == TRX_BUFFER_SIZE){
		unsigned short i = TRX_BUFFER_SIZE - remain;
		for(txb.head=0;txb.head+i<TRX_BUFFER_SIZE;txb.head++){
			txb.buff[txb.head] = txb.buff[txb.head+i];
		}
	}
	
	// 1文字追加
	txb.buff[txb.head++] = c;
	remain++;
	txb.end = txb.head - remain;
	
	// DMACの再設定
	DMA_InitStructure_TX.DMA_MemoryBaseAddr = (uint32_t)(&txb.buff[txb.end]);
	DMA_InitStructure_TX.DMA_BufferSize = remain;
	DMA_Init(DMA1_Channel4, &DMA_InitStructure_TX);
	
	// DMACの動作を許可
	DMA_Cmd(DMA1_Channel4, ENABLE);
	
	// 割り込みマスクの解除
	//mask
}

// 受信バッファから一文字取得
// headに新しく受信データが蓄積され，endから読み出せばいい
// 受信データがあるときはその文字を返す return c
// 受信バッファが空のときはヌル文字を返す return '\0'
volatile char inkey()
{
	// この関数は多重に実行されるとまずいので割り込みをマスクする
	//mask
	
	// headの位置を読み取る
	unsigned short remain = DMA_GetCurrDataCounter(DMA1_Channel5);
	rxb.head = TRX_BUFFER_SIZE - remain;
	
	// バッファが空ならヌルを返す
	if(rxb.head == rxb.end) return '\0';
	
        // バッファから一文字読み取って返す
	char c;
	c = rxb.buff[rxb.end++];
	if(rxb.end == TRX_BUFFER_SIZE) rxb.end = 0;
	
	// 割り込みマスクの解除
	//mask
	
	return c;
}

// 受信バッファから一文字取得
// 受信データが来るまで待機
char inbyte()
{
	char c;
	while((c = inkey())==0);
	return c;
}


//-----------------------------------------------------------------------------
//                           モータ制御
//-----------------------------------------------------------------------------
#define f_MOTOR 20000 // PWM frequency [Hz]
#define T_MOTOR (int(SystemCoreClock/f_MOTOR))
void Motor_Config_TIM1()
{
	// クロックを有効にする
	RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOA | RCC_APB2Periph_AFIO | RCC_APB2Periph_TIM1, ENABLE);

	// PWM出力ポート
	GPIO_StructInit(&GPIO_InitStructure);
	GPIO_InitStructure.GPIO_Pin =  GPIO_Pin_8 | GPIO_Pin_11;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF_PP;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_Init(GPIOA, &GPIO_InitStructure);

	// モータ用タイマ1
	TIM_TimeBaseStructInit(&TIM_TimeBaseStructure);
	TIM_TimeBaseStructure.TIM_Period = T_MOTOR - 1; // 周期の設定
	TIM_TimeBaseStructure.TIM_Prescaler = 0;
	TIM_TimeBaseStructure.TIM_ClockDivision = 0;
	TIM_TimeBaseStructure.TIM_CounterMode = TIM_CounterMode_Up;
	TIM_TimeBaseInit(TIM1, &TIM_TimeBaseStructure);
	// PWMモード
	TIM_OCStructInit(&TIM_OCInitStructure);
	TIM_OCInitStructure.TIM_OCMode = TIM_OCMode_PWM2;
	TIM_OCInitStructure.TIM_OutputState = TIM_OutputState_Enable;
	TIM_OCInitStructure.TIM_Pulse = 0;
	TIM_OCInitStructure.TIM_OCPolarity = TIM_OCPolarity_High;
	TIM_OC1Init(TIM1, &TIM_OCInitStructure);
	TIM_OC4Init(TIM1, &TIM_OCInitStructure);
	TIM_CtrlPWMOutputs(TIM1, ENABLE);

	// タイマ動作開始
	TIM_Cmd(TIM1, ENABLE);
}
void Motor_Config_TIM8()
{
	// クロックを有効にする
	RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOC | RCC_APB2Periph_AFIO | RCC_APB2Periph_TIM8, ENABLE);

	// PWM出力ポート
	GPIO_StructInit(&GPIO_InitStructure);
	GPIO_InitStructure.GPIO_Pin =  GPIO_Pin_6 | GPIO_Pin_7 | GPIO_Pin_8 | GPIO_Pin_9;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF_PP;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_Init(GPIOC, &GPIO_InitStructure);

	// モータ用タイマ8
	TIM_TimeBaseStructInit(&TIM_TimeBaseStructure);
	TIM_TimeBaseStructure.TIM_Period = T_MOTOR - 1; // 周期の設定
	TIM_TimeBaseStructure.TIM_Prescaler = 0;
	TIM_TimeBaseStructure.TIM_ClockDivision = 0;
	TIM_TimeBaseStructure.TIM_CounterMode = TIM_CounterMode_Up;
	TIM_TimeBaseInit(TIM8, &TIM_TimeBaseStructure);
	// PWMモード
	TIM_OCStructInit(&TIM_OCInitStructure);
	TIM_OCInitStructure.TIM_OCMode = TIM_OCMode_PWM2; // PWM1: high-->low, PWM2: low-->high
	TIM_OCInitStructure.TIM_OutputState = TIM_OutputState_Enable;
	TIM_OCInitStructure.TIM_Pulse = 0;
	TIM_OCInitStructure.TIM_OCPolarity = TIM_OCPolarity_High;
	TIM_OC1Init(TIM8, &TIM_OCInitStructure);
	TIM_OC2Init(TIM8, &TIM_OCInitStructure);
	TIM_OC3Init(TIM8, &TIM_OCInitStructure);
	TIM_OC4Init(TIM8, &TIM_OCInitStructure);
	TIM_CtrlPWMOutputs(TIM8, ENABLE);
	
	// タイマ動作開始
	TIM_Cmd(TIM8, ENABLE);
}

void SetDuty_TIM1_14(int duty) // 千分率 [‰]
{
	if(duty > MAXDUTY) duty = MAXDUTY;
	else if(duty < -MAXDUTY) duty = -MAXDUTY;
	duty = duty * T_MOTOR / 1000;
	if(duty >= 0){
		TIM_SetCompare1(TIM1, duty);
		TIM_SetCompare4(TIM1, 0);
	}
	else{
		TIM_SetCompare1(TIM1, 0);
		TIM_SetCompare4(TIM1, -duty);
	}
}
void SetDuty_TIM8_12(int duty) // 千分率 [‰]
{
	if(duty > MAXDUTY) duty = MAXDUTY;
	else if(duty < -MAXDUTY) duty = -MAXDUTY;
	duty = duty * T_MOTOR / 1000;
	if(duty >= 0){
		TIM_SetCompare1(TIM8, duty);
		TIM_SetCompare2(TIM8, 0);
	}
	else{
		TIM_SetCompare1(TIM8, 0);
		TIM_SetCompare2(TIM8, -duty);
	}
}
void SetDuty_TIM8_34(int duty) // 千分率 [‰]
{
	if(duty > MAXDUTY) duty = MAXDUTY;
	else if(duty < -MAXDUTY) duty = -MAXDUTY;
	duty = duty * T_MOTOR / 1000;
	if(duty >= 0){
		TIM_SetCompare3(TIM8, duty);
		TIM_SetCompare4(TIM8, 0);
	}
	else{
		TIM_SetCompare3(TIM8, 0);
		TIM_SetCompare4(TIM8, -duty);
	}
}


//-----------------------------------------------------------------------------
//                           エンコーダ関係
//-----------------------------------------------------------------------------
void Encoder_Config_TIM3()
{
	// クロックを有効にする
	RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOA | RCC_APB2Periph_AFIO, ENABLE);
	RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM3, ENABLE);

	// エンコーダ入力ポート
	GPIO_StructInit(&GPIO_InitStructure);
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_6 | GPIO_Pin_7;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN_FLOATING;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_Init(GPIOA, &GPIO_InitStructure);

	// 位相計数モード
	TIM_EncoderInterfaceConfig(TIM3, TIM_EncoderMode_TI12,TIM_ICPolarity_Rising,TIM_ICPolarity_Rising);

	// タイマ動作開始
	TIM_Cmd(TIM3, ENABLE);
}
void Encoder_Config_TIM4()
{
	// クロックを有効にする
	RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOB | RCC_APB2Periph_AFIO, ENABLE);
	RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM4, ENABLE);

	// エンコーダ入力ポート
	GPIO_StructInit(&GPIO_InitStructure);
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_6 | GPIO_Pin_7;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN_FLOATING;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_Init(GPIOB, &GPIO_InitStructure);

	// 位相計数モード
	TIM_EncoderInterfaceConfig(TIM4, TIM_EncoderMode_TI12,TIM_ICPolarity_Rising,TIM_ICPolarity_Rising);

	// タイマ動作開始
	TIM_Cmd(TIM4, ENABLE);
}
void Encoder_Config_TIM5()
{
	// クロックを有効にする
	RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOA | RCC_APB2Periph_AFIO, ENABLE);
	RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM5, ENABLE);

	// エンコーダ入力ポート
	GPIO_StructInit(&GPIO_InitStructure);
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_0 | GPIO_Pin_1;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN_FLOATING;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_Init(GPIOA, &GPIO_InitStructure);

	// 位相計数モード
	TIM_EncoderInterfaceConfig(TIM5, TIM_EncoderMode_TI12,TIM_ICPolarity_Rising,TIM_ICPolarity_Rising);

	// タイマ動作開始
	TIM_Cmd(TIM5, ENABLE);
}

short encCounter_TIM3(){ return TIM_GetCounter(TIM3); }
short encCounter_TIM4(){ return TIM_GetCounter(TIM4); }
short encCounter_TIM5(){ return TIM_GetCounter(TIM5); }

void SetEncCounter_TIM3(short counter){ TIM_SetCounter(TIM3, counter); }
void SetEncCounter_TIM4(short counter){ TIM_SetCounter(TIM4, counter); }
void SetEncCounter_TIM5(short counter){ TIM_SetCounter(TIM5, counter); }

//-----------------------------------------------------------------------------
//                           入出力ポート
//-----------------------------------------------------------------------------
void IO_Config()
{
	// クロックを有効にする
	RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOA | RCC_APB2Periph_GPIOB | RCC_APB2Periph_GPIOC | RCC_APB2Periph_AFIO, ENABLE);

	// デバッグ用ポートを無効にする
	GPIO_PinRemapConfig(GPIO_Remap_SWJ_Disable, ENABLE);
	
	// 出力ポートの設定
	GPIO_StructInit(&GPIO_InitStructure);
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_10;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP; // push-pull output
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_Init(GPIOC, &GPIO_InitStructure);
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_14 | GPIO_Pin_15;
	GPIO_Init(GPIOA, &GPIO_InitStructure);
}

volatile int enter()
{
	return GPIO_ReadInputDataBit(GPIOB, GPIO_Pin_9);
}


//-----------------------------------------------------------------------------
//                           AD変換
//-----------------------------------------------------------------------------
short ADC_table[nADC*2];
#define f_ADC 16000 // AD変換周波数 [Hz]
#define T_ADC (int(SystemCoreClock/f_ADC))
#define T_CONV (64*20) // 変換にかかる時間
#define SAMPLING_TIME ADC_SampleTime_28Cycles5 // 7,13,28,41,55,71,239
extern void ADC_Config()
{
	// クロックを有効にする
	RCC_AHBPeriphClockCmd(RCC_AHBPeriph_DMA1, ENABLE);
	RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOA | RCC_APB2Periph_GPIOB | RCC_APB2Periph_GPIOC | RCC_APB2Periph_ADC1 | RCC_APB2Periph_ADC2, ENABLE);
	RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM2, ENABLE);

	// AD変換ポート設定
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AIN;
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_0 | GPIO_Pin_1 | GPIO_Pin_2 | GPIO_Pin_4 | GPIO_Pin_5 | GPIO_Pin_6 | GPIO_Pin_7;
	GPIO_Init(GPIOA, &GPIO_InitStructure);
	//GPIO_InitStructure.GPIO_Pin = GPIO_Pin_0 | GPIO_Pin_1;
	//GPIO_Init(GPIOB, &GPIO_InitStructure);
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_0 | GPIO_Pin_1 | GPIO_Pin_2 | GPIO_Pin_3 | GPIO_Pin_4;
	GPIO_Init(GPIOC, &GPIO_InitStructure);

	// 内蔵温度センサ，基準電圧を読めるようにする
	ADC_TempSensorVrefintCmd(ENABLE);

	// ADC1
	ADC_InitStructure.ADC_Mode = ADC_Mode_RegSimult;
	ADC_InitStructure.ADC_ScanConvMode = DISABLE;
	ADC_InitStructure.ADC_ContinuousConvMode = DISABLE;
	ADC_InitStructure.ADC_ExternalTrigConv = ADC_ExternalTrigConv_T2_CC2;
	ADC_InitStructure.ADC_DataAlign = ADC_DataAlign_Right;
	ADC_InitStructure.ADC_NbrOfChannel = nADC;
	ADC_Init(ADC1, &ADC_InitStructure);
	ADC_DiscModeChannelCountConfig(ADC1, 1);
	ADC_DiscModeCmd(ADC1, ENABLE);
	ADC_ExternalTrigConvCmd(ADC1, ENABLE);
	
	// Enable ADC1 DMA
	ADC_DMACmd(ADC1, ENABLE);

	// ADC2
	ADC_InitStructure.ADC_ExternalTrigConv = ADC_ExternalTrigConv_None;
	ADC_Init(ADC2, &ADC_InitStructure);
	ADC_DiscModeChannelCountConfig(ADC2, 1);
	ADC_DiscModeCmd(ADC2, ENABLE);
	ADC_ExternalTrigConvCmd(ADC2, ENABLE); // ADC1と同時サンプリングENABLE

	// 変換順(ADC1)
	ADC_RegularChannelConfig(ADC1, ADC_Channel_17, 1, SAMPLING_TIME); // Vrefint
	ADC_RegularChannelConfig(ADC1, ADC_Channel_13, 2, SAMPLING_TIME); // lf
	ADC_RegularChannelConfig(ADC1, ADC_Channel_13, 3, SAMPLING_TIME); // lf
	ADC_RegularChannelConfig(ADC1, ADC_Channel_10, 4, SAMPLING_TIME); // rf
	ADC_RegularChannelConfig(ADC1, ADC_Channel_10, 5, SAMPLING_TIME); // rf
	ADC_RegularChannelConfig(ADC1, ADC_Channel_1 , 6, SAMPLING_TIME); // 4xOUTZ
	ADC_RegularChannelConfig(ADC1, ADC_Channel_4 , 7, SAMPLING_TIME); // encLA
	ADC_RegularChannelConfig(ADC1, ADC_Channel_7 , 8, SAMPLING_TIME); // encRA
	ADC_RegularChannelConfig(ADC1, ADC_Channel_12, 9, SAMPLING_TIME); // ld
	ADC_RegularChannelConfig(ADC1, ADC_Channel_12,10, SAMPLING_TIME); // ld
	ADC_RegularChannelConfig(ADC1, ADC_Channel_11,11, SAMPLING_TIME); // rd
	ADC_RegularChannelConfig(ADC1, ADC_Channel_11,12, SAMPLING_TIME); // rd
	ADC_RegularChannelConfig(ADC1, ADC_Channel_0 ,13, SAMPLING_TIME); // OUTZ
	ADC_RegularChannelConfig(ADC1, ADC_Channel_1 ,14, SAMPLING_TIME); // 4xOUTZ
	ADC_RegularChannelConfig(ADC1, ADC_Channel_4 ,15, SAMPLING_TIME); // encLA
	ADC_RegularChannelConfig(ADC1, ADC_Channel_7 ,16, SAMPLING_TIME); // encRA

	// 変換順(ADC2)
	ADC_RegularChannelConfig(ADC2, ADC_Channel_6 , 1, SAMPLING_TIME); // Vbatt
	ADC_RegularChannelConfig(ADC2, ADC_Channel_10, 2, SAMPLING_TIME); // rf
	ADC_RegularChannelConfig(ADC2, ADC_Channel_10, 3, SAMPLING_TIME); // rf
	ADC_RegularChannelConfig(ADC2, ADC_Channel_13, 4, SAMPLING_TIME); // lf
	ADC_RegularChannelConfig(ADC2, ADC_Channel_13, 5, SAMPLING_TIME); // lf
	ADC_RegularChannelConfig(ADC2, ADC_Channel_2 , 6, SAMPLING_TIME); // Vref
	ADC_RegularChannelConfig(ADC2, ADC_Channel_5 , 7, SAMPLING_TIME); // encLB
	ADC_RegularChannelConfig(ADC2, ADC_Channel_14, 8, SAMPLING_TIME); // encRB
	ADC_RegularChannelConfig(ADC2, ADC_Channel_11, 9, SAMPLING_TIME); // rd
	ADC_RegularChannelConfig(ADC2, ADC_Channel_11,10, SAMPLING_TIME); // rd
	ADC_RegularChannelConfig(ADC2, ADC_Channel_12,11, SAMPLING_TIME); // ld
	ADC_RegularChannelConfig(ADC2, ADC_Channel_12,12, SAMPLING_TIME); // ld
	ADC_RegularChannelConfig(ADC2, ADC_Channel_2, 13, SAMPLING_TIME); // Vref
	ADC_RegularChannelConfig(ADC2, ADC_Channel_2, 14, SAMPLING_TIME); // Vref
	ADC_RegularChannelConfig(ADC2, ADC_Channel_5, 15, SAMPLING_TIME); // encLB
	ADC_RegularChannelConfig(ADC2, ADC_Channel_14,16, SAMPLING_TIME); // encRB

	// ADCの有効化とキャリブレーション
	ADC_Cmd(ADC1, ENABLE);
	ADC_ResetCalibration(ADC1);
	while(ADC_GetResetCalibrationStatus(ADC1));
	ADC_StartCalibration(ADC1);
	while(ADC_GetCalibrationStatus(ADC1));
	ADC_Cmd(ADC2, ENABLE);
	ADC_ResetCalibration(ADC2);
	while(ADC_GetResetCalibrationStatus(ADC2));
	ADC_StartCalibration(ADC2);
	while(ADC_GetCalibrationStatus(ADC2));

	// regular group用DMAC
	#define ADC1_DR_Address 0x4001244C
	DMA_InitStructure.DMA_PeripheralBaseAddr = (uint32_t)ADC1_DR_Address;
	DMA_InitStructure.DMA_MemoryBaseAddr = (uint32_t)ADC_table;
	DMA_InitStructure.DMA_DIR = DMA_DIR_PeripheralSRC;
	DMA_InitStructure.DMA_BufferSize = nADC;
	DMA_InitStructure.DMA_PeripheralInc = DMA_PeripheralInc_Disable;
	DMA_InitStructure.DMA_MemoryInc = DMA_MemoryInc_Enable;
	DMA_InitStructure.DMA_PeripheralDataSize = DMA_PeripheralDataSize_Word;
	DMA_InitStructure.DMA_MemoryDataSize = DMA_MemoryDataSize_Word;
	DMA_InitStructure.DMA_Mode = DMA_Mode_Circular;
	DMA_InitStructure.DMA_Priority = DMA_Priority_High;
	DMA_InitStructure.DMA_M2M = DMA_M2M_Disable;
	DMA_Init(DMA1_Channel1, &DMA_InitStructure);
	// Enable DMA1 Channel1
	DMA_Cmd(DMA1_Channel1, ENABLE);

	// AD変換トリガタイマ
	TIM_TimeBaseStructInit(&TIM_TimeBaseStructure);
	TIM_TimeBaseStructure.TIM_Period = T_ADC - 1;
	TIM_TimeBaseStructure.TIM_Prescaler =  0;
	TIM_TimeBaseStructure.TIM_ClockDivision = TIM_CKD_DIV1;
	TIM_TimeBaseStructure.TIM_CounterMode = TIM_CounterMode_Up;
	TIM_TimeBaseInit(TIM2, &TIM_TimeBaseStructure);
	// CC2でAD変換
	TIM_OCStructInit(&TIM_OCInitStructure);
	TIM_OCInitStructure.TIM_OCMode = TIM_OCMode_PWM2;
	TIM_OCInitStructure.TIM_OutputState = TIM_OutputState_Enable;
	TIM_OCInitStructure.TIM_Pulse = T_ADC - T_CONV;
	TIM_OCInitStructure.TIM_OCPolarity = TIM_OCPolarity_High;
	TIM_OC2Init(TIM2, &TIM_OCInitStructure);
	
	// タイマ動作開始
	TIM_Cmd(TIM2, ENABLE);
}

//-----------------------------------------------------------------------------
//                            周期タイマ
//-----------------------------------------------------------------------------
volatile float absTime = 0; // [s]
int dutyTimer;
int dutyTimer_max = 0;

void IntervalTimer_Config()
{
	// クロックを有効にする
	RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM6, ENABLE);

	// タイマ設定
	TIM_TimeBaseStructure.TIM_Period = T_INTERVAL - 1; // センサ周期
	TIM_TimeBaseStructure.TIM_Prescaler = SystemCoreClock/1000000 - 1; // 1MHzでカウントアップ
	TIM_TimeBaseStructure.TIM_ClockDivision = TIM_CKD_DIV1;
	TIM_TimeBaseStructure.TIM_CounterMode = TIM_CounterMode_Up;
	TIM_TimeBaseInit(TIM6, &TIM_TimeBaseStructure);

	// NVIC割り込み設定
	NVIC_InitStructure.NVIC_IRQChannel = TIM6_IRQn;
	NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;
	NVIC_InitStructure.NVIC_IRQChannelSubPriority = 1;
	NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
	NVIC_Init(&NVIC_InitStructure);

	TIM_ITConfig(TIM6, TIM_IT_Update, ENABLE); // 割り込み許可

	// センサ用タイマ動作開始
	TIM_Cmd(TIM6, ENABLE);
}

void PreIntRoutine()
{
	TIM_ClearITPendingBit(TIM6, TIM_IT_Update); // 割り込みフラグのクリア

	// 時間の更新
	absTime += dt;
}
void PostIntRoutine()
{
	// 処理の重さの把握
	dutyTimer = TIM_GetCounter(TIM6) * 1000 / T_INTERVAL;
	if(dutyTimer > dutyTimer_max) dutyTimer_max = dutyTimer;
}

void StopIntervalTimer()
{
	TIM_Cmd(TIM6, DISABLE);
}
void RestartIntervalTimer()
{
	TIM_Cmd(TIM6, ENABLE);
}
void StopIntervalInterrupt()
{
	TIM_ITConfig(TIM6, TIM_IT_Update, DISABLE); // 割り込み禁止
}
void RestartIntervalInterrupt()
{
	TIM_ITConfig(TIM6, TIM_IT_Update, ENABLE);
}

void Wait(float time) // [s]
{
	volatile float waitTime;
	waitTime = absTime + time;
	while(waitTime > absTime);
}


//-----------------------------------------------------------------------------
//                           ROMへの書き込み
//-----------------------------------------------------------------------------
volatile FLASH_Status FLASHStatus;
void EraseRom(long addr, long size)
{
	long ptr;
	FLASH_Unlock();
	FLASH_ClearFlag(FLASH_FLAG_BSY | FLASH_FLAG_EOP | FLASH_FLAG_PGERR | FLASH_FLAG_WRPRTERR);
	for(ptr = addr; ptr < addr+size; ptr+=FLASH_PAGE_SIZE){
		FLASHStatus = FLASH_ErasePage(ptr);
	}
	FLASHStatus = FLASH_WaitForLastOperation(0xfff);
}
void WriteToRomWord(long addr, long data)
{
	FLASHStatus = FLASH_ProgramWord(addr, data);
}
